import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;

public class EliminateOutsideOmega {

    /**This routine eliminates a block if it is
       outside the domain Omega from the mongraph
       A return of true means eliminate.
       There are 2 choices for conditions.
       choice=1 is for a non-monotone function like G5^flat
       otherwise choice=0.*/

    public static int main(int choice,Block X) {
       	if(oddConfigTest(X)==true) return 1;
        if(normDisordered(X)==true) return 2;
       	if(normTooBig(X)==true) return 3;
       	if(YCoordDisordered(X)==true) return 4;
	if(choice==0) {if(inHemisphere(X)==true) return 5;}
	return 0;
    }


    /**Eliminates if one of the boxes is outside the disk of radius 3/2 centered at the origin*/
    
    public static boolean normTooBig(Block X) {
	for(int i=1;i<4;++i) {
	   if(normTooBig(X.B[i])==true) return true;
	}
	return false;
    }
    
    public static boolean normTooBig(Box B) {
	long e=1610612736; //3 x 2^29
	GaussianInteger[] G=B.toGaussianVertices();
	for(int i=0;i<4;++i) {
	    if(pythagoreanCompare(G[i],e)==-1) return false;
	}
	return true;
    }
    /**end far out test*/
    
    

    /**checks if the points lie in a halfplane bounded by one of the coord axes*/

    public static boolean inHemisphere(Block X) {
	long x=1;
	for(int i=1;i<4;++i) {
	    long t=X.B[i].xMin();
	    if(x>t) x=t;
	}
	if(x>=0) return true;

	x=1;
	for(int i=1;i<4;++i) {
	    long t=X.B[i].yMin();
	    if(x>t) x=t;
	}
	if(x>=0) return true;
	
	return false;
    }
    /**end hemisphere test*/


    
    /**Checks the the yCoords are correctly ordered*/

    public static boolean YCoordDisordered(Block X) {
        if(X.B[2].yMax()<=0) return true;
	if(X.B[2].yMin()>=X.B[3].yMax()) return true;
	if(X.B[1].yMin()>=X.B[2].yMax()) return true;
	return false;
    }
    /**end Ycoord check*/
    

    /**Returns true if all the configurations in one of the
       boxes B1,B2,B3 have larger norm than all the configurations in B0.
       Actually we check if all points (x,y) in Bk have |x|+|y|>= max B0*/

    public static boolean normDisordered(Block X) {
	long r=X.B[0].xMax();
	for(int i=1;i<4;++i) {
	    if(normDisordered(X.B[i],r)==true) return true;
	}
	return false;
    }

    public static boolean normDisordered(Box B,long r) {
	GaussianInteger[] G=B.toGaussianVertices();
	for(int i=0;i<4;++i) {
	    if(pythagoreanCompare(G[i],r)==-1) return false;
	}
	return true;
    }
    /**end norm disordering tests*/



    

    /**This routine tests whether all configs in the interior of the block are odd*/
    
    public static boolean oddConfigTest(Block X) {
	int a=boxInDiskCount(X);
	if(a%2==0) return false;
	int b=boxOutDiskCount(X);
	if(a+b<4) return false;
	return true;
    }
    
    public static int boxInDiskCount(Block X) {
        long r=536870912;//2^29
	int count=0;
	for(int i=0;i<4;++i) {
  	    if(boxInDisk(X.B[i],r)==true) ++count;
	}
	return count;
    }

    public static boolean boxInDisk(Box B,long r) {
	GaussianInteger[] G=B.toGaussianVertices();
	for(int i=0;i<G.length;++i) {
   	    if(pythagoreanCompare(G[i],r)==+1) return false;
	}
	return true;
    }

    public static int boxOutDiskCount(Block X) {
	long r=536870912;//2^29
	int count=0;
	for(int i=0;i<4;++i) {
	    if(boxOutDisk(X.B[i],r)==true) ++count;
	}
	return count;
    }

    public static boolean boxOutDisk(Box B,long r) {
	GaussianInteger[] G=B.toGaussianVertices();
	for(int i=0;i<G.length;++i) {
	    if(pythagoreanCompare(G[i],r)==-1) return false;
	}
	return true;
    }
    /**end odd configuration test*/


    


    

    /**Does an exact integer multiplication for whether a Gaussian integer has
       norm less than, equal to, or greater than a number c*/
    
    public static int pythagoreanCompare(GaussianInteger G,long c) {
	long a=G.x;
	long b=G.y;
	return pythagoreanCompare(a,b,c);
    }

    /**This routine returns 
       -1 if a^2+b^2<c^2
        0 if a^c+b^2=c^2
       +1 if a^2+b^2>c^2
       We need it because the longs we have
       are too big to multiply out and compare.*/

    

    public static int pythagoreanCompare(long a,long b,long c) {
	Long a0=new Long(a);
	Long b0=new Long(b);
	Long c0=new Long(c);
	BigInteger a1=new BigInteger(a0.toString());
	BigInteger b1=new BigInteger(b0.toString());
	BigInteger c1=new BigInteger(c0.toString());
	BigInteger a2=a1.multiply(a1);
	BigInteger b2=b1.multiply(b1);
	BigInteger c2=c1.multiply(c1);
	BigInteger d=a2.add(b2);
	return d.compareTo(c2);
    }
       


}

