(*This file does the 3 positive dominance tests for Lemma B32 in the monograph*)
(*This is the main step in showing that the FIRST SYMMETRIZATION is energy deceasing*)
(*in the given domain. The first symmetrization is what we call horizontal symmetrization*)


(****************HERE IS THE INTERACTION ENERGY*****************)

(*stereographic projection*)
ST[{x_,y_,z_}]:={x/(1-z),y/(1-z)}
  
  (*Inverse stereographic projection*)
IST[{x_,y_}]:={2 x/(1+x x + y y),2 y/(1+x x + y y),1-2/(1+x x + y y)}

  (*square distance*)
DIST2[v_,w_]:=(v-w).(v-w);

(*1/r^2 potential energy*)
ENERGY2[p_,q_]:=1/DIST2[IST[p],IST[q]];

(**************************************************************)




(**********************HERE IS THE PARAMETRIZATION**************)

f01[t_]:=(416 (1-t) + 498 t)/512;
f02[t_]:=(+16 t)/512
f21[t_]:=(-416 (1-t) - 498 t)/512;
f22[t_]:=(+16 t)/512
f1[t_]:=(348 (1-t) + 465 t)/512;


(**We have to check this for the 4 possibilities S1 = +/- 1 and S2=+/- 1*)

Module[{point0,point1,point2,a1,a3},
       A[a_,b_,c_,d_,e_,S1_,S2_]:=(
                       point0={+f01[a]+49 e/512,S1 f02[b]};
                       point2={+f21[a]+49 e/512,S1 f22[b]};
                       point1={S2 32 d/512,f1[c]};
		       a1=ENERGY2[point0,point1];
		       a3=ENERGY2[point1,point2];
                       a1+a3);]

(*This computes the energy difference*)
B[a_,b_,c_,d_,e_,S1_,S2_]:=A[a,b,c,d,e,S1,S2] - A[a,b,c,0,0,S2,S2];

(************************************************************)



(***********CREATES THE POLYS WE TEST********************)

(*creates the polynomial, the coefficient list, the monomial list*)
polyPP=Expand[Numerator[Together[Expand[B[a,b,c,d,e,+1,+1]]]]]
polyPM=Expand[Numerator[Together[Expand[B[a,b,c,d,e,+1,-1]]]]]
polyMM=Expand[Numerator[Together[Expand[B[a,b,c,d,e,-1,-1]]]]]
polyMP=Expand[Numerator[Together[Expand[B[a,b,c,d,e,-1,+1]]]]]

polyPP1d=D[polyPP,d];
polyPP1e=D[polyPP,e];
directPP=Expand[a polyPP1d + 2 a polyPP1e];
polyPP2d=Expand[a D[polyPP,{d,2}]];
polyPP2e=Expand[a D[polyPP,{e,2}]];

polyPM1d=D[polyPM,d];
polyPM1e=D[polyPM,e];
directPM=Expand[a polyPM1d + 2 a polyPM1e];
polyPM2d=Expand[a D[polyPM,{d,2}]];
polyPM2e=Expand[a D[polyPM,{e,2}]];

polyMM1d=D[polyMM,d];
polyMM1e=D[polyMM,e];
directMM=Expand[a polyMM1d + 2 a polyMM1e];
polyMM2d=Expand[a D[polyMM,{d,2}]];
polyMM2e=Expand[a D[polyMM,{e,2}]];

polyMP1d=D[polyMP,d];
polyMP1e=D[polyMP,e];
directMP=Expand[a polyMP1d + 2 a polyMP1e];
polyMP2d=Expand[a D[polyMP,{d,2}]];
polyMP2e=Expand[a D[polyMP,{e,2}]];

(************END CREATION OF THE POLYS*********************************)


  
(**********5 VARIABLE POSDOM TEST*********************************)

Module[{TRANS5,MAX1,MAX2,MAX3,MAX4,MAX5,coeffs,monomials,exponents,LIST},Posdom5[poly_]:=(
		 coeffs=Table[poly[[j]]/.{a->1,b->1,c->1,d->1,e->1},{j,1,Length[poly]}];
                 monomials=Table[poly[[j]]/coeffs[[j]],{j,1,Length[poly]}];
                 exponents=Table[ExtractExponent5[monomials[[j]]],{j,1,Length[poly]}];

                 (*getting the maximum exponents*)
                 TRANS5=Transpose[exponents];
                 MAX1=Max[TRANS5[[1]]];
                 MAX2=Max[TRANS5[[2]]];
                 MAX3=Max[TRANS5[[3]]];
                 MAX4=Max[TRANS5[[4]]];
                 MAX5=Max[TRANS5[[5]]];
Print["      exponent limits ",MAX1," ",MAX2," ",MAX3," ",MAX4," ",MAX5]; 
                 (*done getting maximum exponents*)

		 (*tailored to the one polynomial we use it on*)
LIST=Table[Table[Table[Table[Table[SumCoeffs[{i1,i2,i3,i4,i5},exponents,coeffs],{i1,0,MAX1}],{i2,0,MAX2}],{i3,0,MAX3}],{i4,0,MAX4}],{i5,0,MAX5}];
                 {Min[Flatten[LIST]],Max[Flatten[LIST]]})];



                 (*supporting routines*)

SumCoeffs[BOUND_,exponents_,coeffs_]:=Sum[SingleTerm[BOUND,exponents,coeffs,k],{k,1,Length[coeffs]}]
SingleTerm[BOUND_,exponents_,coeffs_,k_]:=ExponentDominated[BOUND,exponents[[k]]] coeffs[[k]];
ExponentDominated[BOUND_,small_]:=If[Min[BOUND-small]<0,0,1];
                 

(*extracts the exponent.*)

Module[{val,fac,P},ExtractExponent5[mono_]:=(
		  P=Table[Prime[j],{j,100,105}];
                  val=mono/.{a->P[[1]],b->P[[2]],c->P[[3]],d->P[[4]],e->P[[5]]};
                  fac=FactorInteger[val];
                  Table[ex5[fac,P[[j]]],{j,1,5}])];

Module[{pos},ex5[fac_,p_]:=(
              pos=Position[fac,p];
              If[Length[pos]==0,0,fac[[pos[[1,1]],2]]])];
            
(**************END POSDOM5  METHOD***********************************)




(*************HERE ARE THE MAIN EVALUATIONS*********************************)

Print[Posdom5[directPP]];
Print[Posdom5[polyPP2d]];
Print[Posdom5[polyPP2e]];

Print[Posdom5[directPM]];
Print[Posdom5[polyPM2d]];
Print[Posdom5[polyPM2e]];

Print[Posdom5[directMM]];
Print[Posdom5[polyMM2d]];
Print[Posdom5[polyMM2e]];

Print[Posdom5[directMP]];
Print[Posdom5[polyMP2d]];
Print[Posdom5[polyMP2e]];

(*************END*****************)
