(*This file does the positive dominance test for Lemma B35233.*)
(*This is testing the subdivision 0 of the degree 3 polynomial is weak positive dominant*)
(*This is the most sophisticated test*)
(*In the first part of the program we generate the main polynomial poly5*)
(*In the second part of the program we apply the Amenable discriminant method*)


(*********CREATES THE INTERACTION ENERGY********************)

(*stereographic projection*)
ST[{x_,y_,z_}]:={x/(1-z),y/(1-z)}
  
  (*Inverse stereographic projection*)
IST[{x_,y_}]:={2 x/(1+x x + y y),2 y/(1+x x + y y),1-2/(1+x x + y y)}

  (*square distance*)
DIST2[v_,w_]:=(v-w).(v-w);

(*1/r^2 potential energy*)
ENERGY2[p_,q_]:=1/DIST2[IST[p],IST[q]];

(*********************************************************)



(************THE PARAMETRIZATION***************************)

f01[t_]:=(416 (1-t) + 498 t)/512;
f02[t_]:=(+16 t)/512
f1[t_]:=(348 (1-t) + 465 t)/512;
(*note the minus sign : deliberate*)
g0[t_]:=(-59 t)/512

(***********************************************)




(********ENERGY DIFFERENCE*******************************)

(*This is the basic energy function*)
Module[{point0,point1,point3,a1,a3},
       A[a_,b_,c_,d_]:=(      
          point3={0,+f1[a]+g0[c]};
          point0={f01[b],f02[d]};
          point1={0,-f1[a]+g0[c]};
	  a1=ENERGY2[point0,point1];
          a3=ENERGY2[point0,point3];
          3 a1 a1 + 3 a3 a3 - 4 a1 a3)];

(*This computes the energy difference*)
  B[a_,b_,c_,d_]:=A[a,b,c,d] - A[a,b,0,0];
  (********************************************************)
 

  
(**********5 VARIABLE POSDOM TEST*********************************)
  (*We only need the 3 variable and 4 variable case here but the extra variables do not matter*)
  

Module[{TRANS5,MAX1,MAX2,MAX3,MAX4,MAX5,coeffs,monomials,exponents,LIST},Posdom5[poly_]:=(
		 coeffs=Table[poly[[j]]/.{a->1,b->1,c->1,d->1,e->1},{j,1,Length[poly]}];
                 monomials=Table[poly[[j]]/coeffs[[j]],{j,1,Length[poly]}];
                 exponents=Table[ExtractExponent5[monomials[[j]]],{j,1,Length[poly]}];

                 (*getting the maximum exponents*)
                 TRANS5=Transpose[exponents];
                 MAX1=Max[TRANS5[[1]]];
                 MAX2=Max[TRANS5[[2]]];
                 MAX3=Max[TRANS5[[3]]];
                 MAX4=Max[TRANS5[[4]]];
                 MAX5=Max[TRANS5[[5]]];
Print["      exponent limits ",MAX1," ",MAX2," ",MAX3," ",MAX4," ",MAX5]; 
                 (*done getting maximum exponents*)

		 (*tailored to the one polynomial we use it on*)
LIST=Table[Table[Table[Table[Table[SumCoeffs[{i1,i2,i3,i4,i5},exponents,coeffs],{i1,0,MAX1}],{i2,0,MAX2}],{i3,0,MAX3}],{i4,0,MAX4}],{i5,0,MAX5}];
                 {Min[Flatten[LIST]],Max[Flatten[LIST]]})];



                 (*supporting routines*)

SumCoeffs[BOUND_,exponents_,coeffs_]:=Sum[SingleTerm[BOUND,exponents,coeffs,k],{k,1,Length[coeffs]}]
SingleTerm[BOUND_,exponents_,coeffs_,k_]:=ExponentDominated[BOUND,exponents[[k]]] coeffs[[k]];
ExponentDominated[BOUND_,small_]:=If[Min[BOUND-small]<0,0,1];
                 

(*extracts the exponent.*)

Module[{val,fac,P},ExtractExponent5[mono_]:=(
		  P=Table[Prime[j],{j,100,105}];
                  val=mono/.{a->P[[1]],b->P[[2]],c->P[[3]],d->P[[4]],e->P[[5]]};
                  fac=FactorInteger[val];
                  Table[ex5[fac,P[[j]]],{j,1,5}])];

Module[{pos},ex5[fac_,p_]:=(
              pos=Position[fac,p];
              If[Length[pos]==0,0,fac[[pos[[1,1]],2]]])];
            
(**************END POSDOM5  METHOD***********************************)


  
  
  
  (*************CREATING THE BASIC POLYNOMIAL**************)
(*creates the polynomial to be tested*)
    poly2=Expand[Numerator[Together[Expand[B[a,b,c,d]]]]];
    poly2a=Expand[a D[poly2,a]];
    poly3=poly2//.a->0;

  (*now we take the 1st subdivision S_{d,1}(poly4)*)
  poly4=poly3//.d->e;
  poly5=poly4//.e->1/2 d+1/2;

  (*********************************************************)
  

  
  
  
  (***************THE TEST*******************)
Print[Posdom5[poly2a]];
Print[Posdom5[poly5]];
