(*This file does the positive dominance test for Lemma B35233.*)
(*This is testing the subdivision 0 of the degree 3 polynomial is weak positive dominant*)
(*This is the most sophisticated test*)
(*In the first part of the program we generate the main polynomial poly5*)
(*In the second part of the program we apply the Amenable discriminant method*)


(*********CREATES THE INTERACTION ENERGY********************)

(*stereographic projection*)
ST[{x_,y_,z_}]:={x/(1-z),y/(1-z)}
  
  (*Inverse stereographic projection*)
IST[{x_,y_}]:={2 x/(1+x x + y y),2 y/(1+x x + y y),1-2/(1+x x + y y)}

  (*square distance*)
DIST2[v_,w_]:=(v-w).(v-w);

(*1/r^2 potential energy*)
ENERGY2[p_,q_]:=1/DIST2[IST[p],IST[q]];

(*********************************************************)



(************THE PARAMETRIZATION***************************)

f01[t_]:=(416 (1-t) + 498 t)/512;
f02[t_]:=(+16 t)/512
f1[t_]:=(348 (1-t) + 465 t)/512;
(*note the minus sign : deliberate*)
g0[t_]:=(-59 t)/512

(***********************************************)




(********ENERGY DIFFERENCE*******************************)

(*This is the basic energy function*)
Module[{point0,point1,point3,a1,a3},
       A[a_,b_,c_,d_]:=(      
          point3={0,+f1[a]+g0[c]};
          point0={f01[b],f02[d]};
          point1={0,-f1[a]+g0[c]};
	  a1=ENERGY2[point0,point1];
          a3=ENERGY2[point0,point3];
          3 a1 a1 + 3 a3 a3 - 4 a1 a3)];

(*This computes the energy difference*)
  B[a_,b_,c_,d_]:=A[a,b,c,d] - A[a,b,0,0];
  (********************************************************)
 

  
  
  
  (*************CREATING THE BASIC POLYNOMIAL**************)
(*creates the polynomial to be tested*)
    poly2=Expand[Numerator[Together[Expand[B[a,b,c,d]]]]];
    poly3=poly2//.a->0;

  (*now we take the 0th subdivision S_{d,0}(poly4)*)
  poly4=poly3//.d->e;
  poly5=poly4//.e->1/2 d;

  (*********************************************************)
  
  
  
  (********THE AMENABLE DISCRIMINANT***************************)
  
  (*At this point we have the polynomial poly5. It is a 3-variable polynomial*)
  (*in the variables b,c,d*)
  (*These are special routines needed to compute the Amenable discriminant*)


  (*Now we kill off higher order terms in c,d.  We pay a quadratic*)
  (*penalty each time we do this.  This is explained in the monograph*)
  (*in connection with LemmaB35233*)

  Module[{X0,X1,X2,X3}, 
         PolynomialInfluence[X_]:=(
            X0=Table[X[[j]],{j,1,Length[X]}];
	    X1=X0//.{b->1,c->1,d->1};
            X2=Table[Abs[-1+Sign[X1[[j]]]]/2,{j,1,Length[X1]}];
	    X3=-X1.X2)];


  GetHigherTerms[P_,h1_,h2_]:=Expand[Sum[Sum[Power[c,i1] Power[d,i2] Coefficient[Coefficient[P,c,i1],d,i2],{i1,h1,20}],{i2,h2,20}]];
  
  Module[{P0},
         KillHigherTerms[P_,h1_,h2_,k1_,k2_]:=(
           P0=GetHigherTerms[P,h1,h2];
           Expand[P-P0-PolynomialInfluence[P0] Power[c,k1] Power[d,k2]])];

    poly6=KillHigherTerms[poly5,3,0,2,0];
    poly7=KillHigherTerms[poly6,0,3,0,2];
    poly8=KillHigherTerms[poly7,2,1,1,1];
    poly9=KillHigherTerms[poly8,1,2,1,1];

(*poly9 is the output of this process.  Now we compute the quadratish discriminant*)

pA=Coefficient[poly9,c,2];
pB=Coefficient[Coefficient[poly8,c,1],d,1];
pC=Coefficient[poly9,d,2];

(**Here is the quadratish discriminant*)

DISC=Expand[4 pA pC - pB pB];
(****************************************************************)


(****************1 VARIABLE POSITIVE DOMINANCE TEST***************)
SumTerm[P_,k_]:=Sum[Coefficient[P,t,j],{j,0,k}]
Posdom1[P_]:=Sign[Table[SumTerm[P,k],{k,0,20}]]
(**************************************************************)

(**************THE TEST********************************)

(**DISC is not positive dominant but the two subdivisions are*)
 
DISC0=Expand[DISC//.b->t/2];
DISC1=Expand[DISC//.b->t/2+1/2];
Print[Posdom1[DISC0]];
Print[Posdom1[DISC1]];

(*****************************************************************)