import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.math.*;


public class Energy {

    
    /**Here is the controlled power function
       sign=-1 gives an under-approximation and
       sign=+1 gives an over-approximation.  The routine
       uses floating point arithmetic to make a guess. Then,
       after the guess is made, it is rigorously checked
       using exact arithmetic.  The checker is very slow.*/

    public static BigRational pow(int sign,BigRational A,BigRational E) {
	double a=A.toDouble();
	double e=E.toDouble();
	double x=Math.pow(a,e);
	double u=Math.pow(2,32)*x;
	long n=(long)(u);
	Long N=new Long(n+2*sign);
	BigRational X=new BigRational(N.toString(),"4294967296");// 2^32
	X.reduce();
	int test=BigRational.testPow(A,E,X);
	if(test!=sign) throw new ProofException("problem with power function");
	return X;
    }

    /**Here is the main routine.  Theta is the function from the paper.
       We are computing a lower bound to theta.   The evaluation is at
       (s,x,y) where
       s=s/65536
       x=X/65536,
       y=Y/65536)*/

    public static BigRational theta(long S,long X,long Y) {
	BigRational d0=energyUnder(S,X,Y);
	BigRational d1=energyTBPOver(S);
	return d0.subtract(d1);
    }
    
    public static  BigRational energyUnder(long S,long X,long Y) {
	BigRational[] s=energyTermsUnder(S,X,Y);
	BigRational t=new BigRational(0,1);
	for(int i=0;i<10;++i) t=t.add(s[i]);
	return t;
    }


    public static  BigRational[] energyTermsUnder(long S,long X,long Y) {
	BigRational t1=A(-1,S,X);
	BigRational t2=A(-1,S,Y);
	BigRational t3=C(-1,S,X,Y);
	BigRational t4=B(-1,S,X);
	BigRational t5=B(-1,S,Y);
	BigRational[] t={t1,t2,t4,t4,t5,t5,t3,t3,t3,t3};
 	return t;
    }

    /**Here are the component functions which go into the final
       calculation of the energy.  Again, the exponent is
       given by s=S/65536.   We take (S/65536)/2=S/131072 so
       as to match the fact that we the function calls for
       s/2 rather than s.  See the formulas in the 
       EnergyFloat class.  The debugger (in the main program)
       checks that this is the correct thing to do.*/

    public static  BigRational A(int sign,long S,long X) {
	BigRational s=new BigRational(S,131072);
	BigRational R=pow(sign,a(X),s);
	return R;
    }

    public static  BigRational B(int sign,long S,long X) {
	BigRational s=new BigRational(S,131072);
	BigRational R=pow(sign,b(X),s);
	return R;
    }

    public static  BigRational C(int sign,long S,long X,long Y) {
	BigRational s=new BigRational(S,131072);
	BigRational R=pow(sign,c(X,Y),s);
	return R;
    }


    public static  BigRational a(long X) {
	BigRational BX=new BigRational(X,65536);
	BigRational ONE=new BigRational(1,1);
	BigRational A=ONE.add(BX.multiply(BX));
	A=A.multiply(A);
	A=A.divide(BX);
	A=A.divide(BX);
	A=A.multiply(new BigRational(1,16));
	return A;
    }

    public static  BigRational b(long X) {
	BigRational BX=new BigRational(X,65536);
	BigRational ONE=new BigRational(1,1);
	BigRational A=ONE.add(BX.multiply(BX));
	A=A.multiply(new BigRational(1,4));
	return A;
    }

    public static  BigRational c(long X,long Y) {
	BigRational BX=new BigRational(X,65536);
	BigRational BY=new BigRational(Y,65536);
	BigRational ONE=new BigRational(1,1);
	BigRational A=ONE.add(BX.multiply(BX));
	BigRational B=ONE.add(BY.multiply(BY));
	A=A.multiply(B);
	BigRational C=BX.multiply(BX);
	C=C.add(BY.multiply(BY));
	A=A.divide(C);
	A=A.multiply(new BigRational(1,4));
	return A;
    }


    
    /**Here are the analogous routines for the TBP*/

    public static  BigRational energyTBPOver(long S) {
	BigRational[] s=energyTermsTBPOver(S);
	BigRational t=new BigRational(0,1);
	for(int i=0;i<10;++i) t=t.add(s[i]);
	return t;
    }

    public static  BigRational[] energyTermsTBPOver(long S) {
	BigRational X2=new BigRational(1,2);
	BigRational X3=new BigRational(1,3);
	BigRational X4=new BigRational(1,4);
	BigRational s=new BigRational(S,131072);
	BigRational t2=pow(1,X2,s);
	BigRational t3=pow(1,X3,s);
	BigRational t4=pow(1,X4,s);
	BigRational[] t={t2,t2,t2,t2,t2,t2,t3,t3,t3,t4};
 	return t;
    }

    

    

}


    
